//
//
// Copyright (C) 2004, 2005 Pingtel Corp.
// 
//
// $$
////////////////////////////////////////////////////////////////////////
//////


#ifndef _OsDatagramSocket_h_
#define _OsDatagramSocket_h_

// SYSTEM INCLUDES
//#include <...>

// APPLICATION INCLUDES
#include <os/OsSocket.h>

// DEFINES
// MACROS
// EXTERNAL FUNCTIONS
// EXTERNAL VARIABLES
// CONSTANTS
// STRUCTS
// TYPEDEFS
// FORWARD DECLARATIONS

//: Implements UDP version of OsSocket
// This class provides the implementation of the UDP datagram-based
// socket class which may be instantiated.

class OsDatagramSocket : public OsSocket
{
/* //////////////////////////// PUBLIC //////////////////////////////////// */
public:

/* ============================ CREATORS ================================== */
   OsDatagramSocket(sipx_int_t remoteHostPort, const char* remoteHostName,
                    sipx_int_t localHostPort = PORT_DEFAULT,
                    const char* localHostName = NULL);

  virtual
   ~OsDatagramSocket();
     //:Destructor


/* ============================ MANIPULATORS ============================== */

   virtual UtlBoolean reconnect();
   //: Sets up the connection again, assuming the connection failed

   void doConnect(sipx_int_t remotePort, const char* remoteHostName,
       UtlBoolean simulateConnect = FALSE);
   // Setup a connection with a remote host on the specified remote port
   //! param: remoteHostName - remote host to send datagram(s) in subsequent calls to write (overloaded version without host and port)
   //! param: remotePort - port on the remote host to send the datgram(s)
   //! param: simulateConnect - if TRUE do not call connect but default the to location (as in sendto) to the given host and port.  This is desirable to prevent the receiving of packets from other hosts from being filtered out by the IP stack.

   virtual sipx_int_t write(const char* buffer, sipx_int_t bufferLength);
   //: Blocking write to the socket
   // Write the characters in the given buffer to the socket.
   // This method will block until all of the bytes are written.
   //! param: buffer - the bytes to be written to the socket
   //! param: bufferLength - the number of bytes contained in buffer
   //! returns: the number of bytes actually written to the socket

   virtual sipx_int_t write(const char* buffer, sipx_int_t bufferLength,
       const char* ipAddress, sipx_int_t port);
   //: Blocking write to the socket
   // Write the characters in the given buffer to the socket.
   // This method will block until all of the bytes are written.
   //! param: buffer - the bytes to be written to the socket
   //! param: bufferLength - the number of bytes contained in buffer
   //! param: ipAddress - remote host to send datagram(s) to
   //! param: port - port on the remote host to send the datgram(s)
   //! returns: the number of bytes actually written to the socket

   virtual sipx_int_t read(char* buffer, sipx_int_t bufferLength);

/* ============================ ACCESSORS ================================= */
   virtual OsSocket::IpProtocolSocketType getIpProtocol() const;
   //: Returns the protocol type of this socket


   virtual void getRemoteHostIp(struct in_addr* remoteHostAddress,
                        sipx_int_t* remotePort = NULL);
   //:Return remote host ip address
   // Returns the ip address for the host on which the socket on the
   // other end of this socket is bound.

   virtual UtlBoolean getExternalIp(UtlString* ip, sipx_int_t* port) ;
     //:Return the external IP address for this socket.
     // OsStunDatagramSocket will return the NATted address if available,

/* ============================ INQUIRY =================================== */

/* //////////////////////////// PROTECTED ///////////////////////////////// */
protected:

/* //////////////////////////// PRIVATE /////////////////////////////////// */
private:
   OsDatagramSocket(const OsDatagramSocket& rOsDatagramSocket);
     //:Disable copy constructor

   OsDatagramSocket();
     //:Disable default constructor

   OsDatagramSocket& operator=(const OsDatagramSocket& rhs);
     //:Assignment operator

   time_t mLastWriteErrorTime;
   sipx_int_t    mNumTotalWriteErrors;
   sipx_int_t    mNumRecentWriteErrors;

   UtlBoolean mSimulatedConnect;
   UtlBoolean mToSockaddrValid;
   struct sockaddr_in* mpToSockaddr;

   virtual sipx_int_t writeTo(const char* buffer, sipx_int_t bufferLength);
   //: Blocking write to the socket, with simulated connection
   // Write the characters in the given buffer to the socket.
   // This method will block until all of the bytes are written.
   //! param: buffer - the bytes to be written to the socket
   //! param: bufferLength - the number of bytes contained in buffer
   //! returns: the number of bytes actually written to the socket

   virtual UtlBoolean getToSockaddr(void);

};

/* ============================ INLINE METHODS ============================ */

#endif  // _OsDatagramSocket_h_
