//
// Copyright (C) 2004, 2005 Pingtel Corp.
// 
//
// $$
////////////////////////////////////////////////////////////////////////
//////


#ifndef _OsProtectEventMgr_h_
#define _OsProtectEventMgr_h_

// SYSTEM INCLUDES

// APPLICATION INCLUDES
#include "os/OsDefs.h"
#include "os/OsBSem.h"
#include "os/OsProtectEvent.h"

// DEFINES
// MACROS
// EXTERNAL FUNCTIONS
// EXTERNAL VARIABLES
// CONSTANTS
// STRUCTS
// TYPEDEFS
// FORWARD DECLARATIONS


class OsProtectEventMgr
{

/* //////////////////////////// PUBLIC //////////////////////////////////// */
public:

/* ============================ CREATORS ================================== */

   static OsProtectEventMgr* getEventMgr(sipx_int_t userData = 0);
     //:Return a pointer to the OsProtectEventMgr, creating it if necessary

   virtual
   ~OsProtectEventMgr();
     //:Destructor

/* ============================ MANIPULATORS ============================== */

   OsProtectedEvent* alloc(sipx_int_t userData = 0);
     //:Allocate a new OsProtectEvent
     // Return OS_SUCCESS if allocation is successful

   OsStatus release(OsProtectedEvent* pEvent);
     //:Release the OsProtectEvent
     // Return OS_SUCCESS if the OsProtectEvent is released.

/* ============================ ACCESSORS ================================= */

   sipx_int_t allocCnt();
     //:Return the number of events that are allocated.

   sipx_int_t availCnt();
     //:Return the number of events that are available.

/* ============================ INQUIRY =================================== */

/* //////////////////////////// PROTECTED ///////////////////////////////// */
protected:

#if defined(_pingtel_on_posix_)
        OsProtectEventMgr(sipx_int_t userData = 0, sipx_int_t initialCount = 1000,
                                        sipx_int_t softLimit = 2500,
                                        sipx_int_t hardLimit = 25000,
                                        sipx_int_t increment = 500);
#else
        OsProtectEventMgr(sipx_int_t userData = 0, sipx_int_t initialCount = 200,
                                        sipx_int_t softLimit = 500,
                                        sipx_int_t hardLimit = 5000,
                                        sipx_int_t increment = 50);
#endif
     //:Constructor

/* //////////////////////////// PRIVATE /////////////////////////////////// */
private:

   // Static data members used to enforce Singleton behavior
   static OsProtectEventMgr*    spInstance;             // pointer to the single instance of
                                                                                                                                //  the OsProtectEventMgr class
   static OsBSem     sLock;         // semaphore used to ensure that there
                                    //  is only one instance of this class

   OsBSem    mListSem;   // semaphore used to protect the list table
   sipx_int_t       mAllocs;    // number of allocations
   sipx_int_t       mFrees;    // number of freed events

   sipx_int_t      mInitialCount;   // initial number of items
   sipx_int_t      mCurrentCount;   // current number of items
   sipx_int_t      mSoftLimit;      // soft limit, warn when expanding above this
   sipx_int_t      mHardLimit;      // hard limit AND length of mpElts
   sipx_int_t      mIncrement;      // number to create when expanding
   sipx_int_t      mNext;           // index to next element to examine

   OsProtectedEvent**  mpEvents;          // array of pointers to contained objects

   OsProtectEventMgr(const OsProtectEventMgr& rOsProtectEventMgr);
     //:Copy constructor (not implemented for this class)

   OsProtectEventMgr& operator=(const OsProtectEventMgr& rhs);
     //:Assignment operator (not implemented for this class)

        OsStatus addtoList(sipx_uint_t key, sipx_uint_t pEvent);

        OsStatus remove(sipx_uint_t key);

};

/* ============================ INLINE METHODS ============================ */

#endif  // _OsProtectEventMgr_h_
