//
// Copyright (C) 2004, 2005 Pingtel Corp.
// 
//
// $$
////////////////////////////////////////////////////////////////////////
//////


#ifndef _OsTime_h_
#define _OsTime_h_

// SYSTEM INCLUDES

// APPLICATION INCLUDES
#include "os/OsDefs.h"
#include "utl/UtlDefs.h"

// DEFINES
// MACROS
// EXTERNAL FUNCTIONS
// EXTERNAL VARIABLES
// CONSTANTS
// STRUCTS
// TYPEDEFS
// FORWARD DECLARATIONS

//:Time or time sipx_int_terval
// If necessary, this class will adjust the seconds and microseconds values
// that it reports such that 0 <= microseconds < USECS_PER_SEC.


class OsTime
{
/* //////////////////////////// PUBLIC //////////////////////////////////// */
public:
   static const OsTime OS_INFINITY;
   static const OsTime NO_WAIT;
   static const sipx_long_t MSECS_PER_SEC ; 
   static const sipx_long_t USECS_PER_MSEC ;
   static const sipx_long_t USECS_PER_SEC ;

/* ============================ CREATORS ================================== */

   OsTime();
     //:Default constructor (creates a zero duration sipx_int_terval)

   OsTime(const sipx_long_t msecs);
     //:Constructor specifying time/duration in terms of milliseconds

   OsTime(const sipx_long_t seconds, const sipx_long_t usecs);
     //:Constructor specifying time/duration in terms of seconds and microseconds

   OsTime(const OsTime& rOsTime);
     //:Copy constructor

   virtual
   ~OsTime();
     //:Destructor

/* ============================ MANIPULATORS ============================== */

   OsTime& operator=(const OsTime& rhs);
     //:Assignment operator

   OsTime operator+(const OsTime& rhs);
     //:Addition operator

   OsTime operator-(const OsTime& rhs);
     //:Subtraction operator

   OsTime operator+=(const OsTime& rhs);
     //:Increment operator

   OsTime operator-=(const OsTime& rhs);
     //:Decrement operator

   bool operator==(const OsTime& rhs);
     //:Test for equality operator

   bool operator!=(const OsTime& rhs);
     //:Test for inequality operator

   bool operator>(const OsTime& rhs);
     //:Test for greater than

   bool operator>=(const OsTime& rhs);
     //:Test for greater than or equal

   bool operator<(const OsTime& rhs);
     //:Test for less than

   bool operator<=(const OsTime& rhs);
     //:Test for less than or equal

/* ============================ ACCESSORS ================================= */

   virtual sipx_long_t seconds(void) const;
     //:Return the seconds portion of the time sipx_int_terval

   virtual sipx_long_t usecs(void) const;
     //:Return the microseconds portion of the time sipx_int_terval

   virtual sipx_long_t cvtToMsecs(void) const;
     //:Convert the time sipx_int_terval to milliseconds

/* ============================ INQUIRY =================================== */

   virtual UtlBoolean isInfinite(void) const;
     //:Return TRUE if the time sipx_int_terval is infinite

   virtual UtlBoolean isNoWait(void) const;
     //:Return TRUE if the time sipx_int_terval is zero (no wait)

/* //////////////////////////// PROTECTED ///////////////////////////////// */
protected:

/* //////////////////////////// PRIVATE /////////////////////////////////// */
private:
   sipx_long_t mSeconds;
   sipx_long_t mUsecs;

   void init(void);
     //:Initialize the instance variables for a newly constructed object

};

/* ============================ INLINE METHODS ============================ */

#endif  // _OsTime_h_

