// 
// 
// Copyright (C) 2005-2006 SIPez LLC.
// Licensed to SIPfoundry under a Contributor Agreement.
// 
// Copyright (C) 2004-2006 SIPfoundry Inc.
// Licensed by SIPfoundry under the LGPL license.
// 
// Copyright (C) 2004-2006 Pingtel Corp.
// Licensed to SIPfoundry under a Contributor Agreement.
// 
// $$
//////////////////////////////////////////////////////////////////////////////


#ifndef _UtlLongLongInt_h_
#define _UtlLongLongInt_h_

// SYSTEM INCLUDES
// APPLICATION INCLUDES
#include "utl/UtlDefs.h"
#include "utl/UtlContainable.h"

// DEFINES
// MACROS

// Borrow this from the gcc include file "limits.h".
// LLONG_MIN and LLONG_MAX are defined by the ISO C99 standard but not by C++
// LONG_LONG_MIN and LONG_LONG_MAX are defined by gcc.
#ifndef LLONG_MIN
# define LLONG_MIN	LONG_LONG_MIN
#endif
#ifndef LLONG_MAX
# define LLONG_MAX	LONG_LONG_MAX
#endif

// EXTERNAL FUNCTIONS
// EXTERNAL VARIABLES
// CONSTANTS
// STRUCTS
// TYPEDEFS
// FORWARD DECLARATIONS

/**
 * UtlLongLongInt is a UtlContainable wrapper for a "sipx_longlong_t".
 * This data type is officially part of C99, see http://www.open-std.org/jtc1/sc22/wg14/ .
 * C++ has de facto support as well.  The size of a "sipx_longlong_t" is guaranteed to be at least 64 bits.
 */
class UtlLongLongInt : public UtlContainable
{
/* //////////////////////////// PUBLIC //////////////////////////////////// */
public:

/* ============================ CREATORS ================================== */

    /**
     * Constructor accepting an optional default value.
     */
    UtlLongLongInt(INT64 initialValue = 0) ;
      
    /**
     * Destructor
     */
    virtual ~UtlLongLongInt();

/* ============================ OPERATORS ============================== */

    // Declare prefix and postfix increment operators.
    UtlLongLongInt& operator++();       // Prefix increment operator
    UtlLongLongInt operator++(int);     // Postfix increment operator

    // Declare prefix and postfix decrement operators.
    UtlLongLongInt& operator--();       // Prefix decrement operator
    UtlLongLongInt operator--(int);     // Postfix decrement operator

    // Conversion to sipx_longlong_t
    operator INT64() { return mValue; }

/* ============================ MANIPULATORS ============================== */

    /**
     * Set a new sipx_longlong_t value for this object.
     *
     * @returns the old value
     */
    INT64 setValue(INT64 iValue);

    //! Convert a ascii string rep. to sipx_longlong_t
    static INT64 stringToLongLong(const char* sipx_long_tLongString);

/* ============================ ACCESSORS ================================= */

    /**
     * Get the sipx_longlong_t wrapped by this object.
     */
    INT64 getValue() const ;    

    /**
     * Calculate a unique hash code for this object.  If the equals
     * operator returns true for another object, then both of those
     * objects must return the same hashcode.
     */
    virtual sipx_uint_t hash() const ;

    /**
     * Get the ContainableType for a UtlContainable derived class.
     */
    virtual UtlContainableType getContainableType() const;

    static UtlContainableType TYPE ;    /**< Class type used for runtime checking */ 

/* ============================ INQUIRY =================================== */

    /**
     * Compare the this object to another like-object.  Results for 
     * designating a non-like object are undefined.
     *
     * @returns 0 if equal, < 0 if less then and >0 if greater.
     */
    virtual sipx_int_t compareTo(UtlContainable const *) const ;    

    /**
     * Test this object to another like-object for equality.  This method 
     * returns false if unlike-objects are specified.
     */
    virtual UtlBoolean isEqual(UtlContainable const *) const ; 

/* //////////////////////////// PROTECTED ///////////////////////////////// */
protected:

/* //////////////////////////// PRIVATE /////////////////////////////////// */
private:
    INT64 mValue ;    /** < The sipx_longlong_t wrapped by this object */ 

} ;

/* ============================ INLINE METHODS ============================ */

#endif    // _UtlLongLongInt_h_
