//
// Copyright (C) 2004, 2005 Pingtel Corp.
// 
//
// $$
////////////////////////////////////////////////////////////////////////
//////


#ifndef _UtlRscStore_h_
#define _UtlRscStore_h_

#include "utl/UtlRscTrace.h"

#ifdef RSC_TEST


// SYSTEM INCLUDES
#include "os/OsDefs.h"
#include "os/OsBSem.h"
#include "os/OsRWMutex.h"
#include "os/OsStatus.h"

// APPLICATION INCLUDES
// DEFINES
// MACROS
// EXTERNAL FUNCTIONS
// EXTERNAL VARIABLES
// CONSTANTS
// STRUCTS
// TYPEDEFS
// FORWARD DECLARATIONS
class UtlRscTrace;

//:Database of active Rscs.
// The UtlRscStore masipx_int_tains a database of active Rscs (i.e., those Rscs
// that have been started by the low level OsSysRsc class).  Since the
// OsRscTask is the only task that should be accessing the Rsc database
// there is no need to serialize access (and no locking).<br>
// <br>
// Each entry in the database is a key/value pair where the key corresponds
// to a Rsc ID and the value is the pointer to the corresponding OsRsc
// object.  Duplicate keys are not allowed.
class UtlRscStore
{
friend UtlRscTrace;

/* //////////////////////////// PUBLIC //////////////////////////////////// */
public:

   enum StoreInitSize { RSC_STORE_INIT_SIZE = 1000000 };

/* ============================ CREATORS ================================== */

   UtlRscStore(sipx_int_t initialStoreSize = RSC_STORE_INIT_SIZE);
     //:Default constructor

   virtual
   ~UtlRscStore();
     //:Destructor

/* ============================ MANIPULATORS ============================== */

   OsStatus insert(sipx_int_t RscId, char* pRsc);
     //:Insert the indicated Rsc sipx_int_to the database of active Rscs.
     // Return OS_SUCCESS if successful, OS_NAME_IN_USE if the key is
     // already in the database.

   OsStatus remove(sipx_int_t RscId);
     //:Remove the indicated Rsc from the database of active Rscs.
     // Return OS_SUCCESS if the indicated RscId is found, return
     // OS_NOT_FOUND if there is no match for the specified key.

   void cleanUp();

/* ============================ ACCESSORS ================================= */

   sipx_int_t getActiveRscs(char* activeRscs[], sipx_int_t size);
     //:Get an array of pointers to the Rscs that are currently active.
     // The caller provides an array that can hold up to <i>size</i> OsRsc
     // pointers. This method will fill in the <i>activeRscs</i> array with
     // up to <i>size</i> pointers. The method returns the number of pointers
     // in the array that were actually filled in.

   void getStoreStats(sipx_uint_t& nInserts, sipx_uint_t& nRemoves) const;
     //:Get the number of insertions and removals for the Rsc database.

   sipx_int_t numEntries(void) const;
     //:Return the number of key-value pairs in the name database.

/* ============================ INQUIRY =================================== */

   UtlBoolean isEmpty(void) const;
     //:Return TRUE if the Rsc database is empty.

/* //////////////////////////// PROTECTED ///////////////////////////////// */
protected:

/* //////////////////////////// PRIVATE /////////////////////////////////// */
private:
   UtlHashMap mDict;          // hash table used to store the key/value
                                    //  pairs
   sipx_ulong_t mNumInserts;            // number of insertions sipx_int_to the database
   sipx_ulong_t mNumRemoves;            // number of removals from the database

   OsRWMutex mDictRWLock;
   UtlRscStore(const UtlRscStore& rUtlRscStore);
     //:Copy constructor (not implemented for this class)

   UtlRscStore& operator=(const UtlRscStore& rhs);
     //:Assignment operator (not implemented for this class)

#ifdef TEST
   static bool sIsTested;
     //:Set to true after the tests for this class have been executed once

   void test();
     //:Verify assertions for this class

   // Test helper functions
   void testCreators();
   void testManipulators();
   void testAccessors();
   void testInquiry();

#endif TEST
};

/* ============================ INLINE METHODS ============================ */
#endif // RSC_TEST

#endif  // _UtlRscStore_h_

