//
// Copyright (C) 2004, 2005 Pingtel Corp.
// 
//
// $$
////////////////////////////////////////////////////////////////////////
//////


#ifndef _UtlRscTrace_h_
#define _UtlRscTrace_h_

#define RSC_TEST
#undef RSC_TEST

#ifdef RSC_TEST

// SYSTEM INCLUDES
#include "utl/UtlRscStore.h"
#include <os/OsMutex.h>

// APPLICATION INCLUDES
// EXTERNAL FUNCTIONS
// EXTERNAL VARIABLES
// CONSTANTS
// STRUCTS
// TYPEDEFS
// FORWARD DECLARATIONS

//:A class used to keep track of the number of resource allocations and frees
// This class is used in conjunction with instrumented versions of the global
// new and delete operators.
class UtlRscTrace
{
/* //////////////////////////// PUBLIC //////////////////////////////////// */
public:
   static sipx_ulong_t sAllocCnt; // Track the number of memory allocs
   static sipx_ulong_t sFreeCnt;  // Track the number of memory frees
   static sipx_int_t           sTraceFlag;// If non-zero, print a line of info
                                   //  for every call to new()/delete()
   static sipx_ulong_t sStartTime;  // 
   static UtlRscStore mResourceStore;

   enum RscType
   {
          NONE          = 0,
      MEMORY    = 1,   
      OSSOCKET  = 2,       
      OSMSGQ    = 3,  
      OSBSEM    = 4,
          OSCSEM        = 5,
          OSMUTEX       = 6,
          OSRWMUTEX     = 7,
          OSTIMER       = 8,
          OSTASK        = 9
   };


/* ============================ CREATORS ================================== */

   UtlRscTrace();
     //:Default constructor (and take a checkpoint)

   UtlRscTrace(const UtlRscTrace& rRscTrace);
     //:Copy constructor

   virtual
   ~UtlRscTrace();
     //:Destructor

/* ============================ MANIPULATORS ============================== */

   UtlRscTrace& operator=(const UtlRscTrace& rhs);
     //:Assignment operator

   virtual void checkpoint();
     //:Remember the count of outstanding memory allocations

   static void setTraceFlag(sipx_int_t flag) { sTraceFlag = flag; };
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t enableMemTracking(sipx_int_t taskId = 0);
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t enableMsgQTracking(sipx_int_t taskId = 0);
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t enableBSemTracking(sipx_int_t taskId = 0);
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t enableCSemTracking(sipx_int_t taskId = 0);
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t enableMutexTracking(sipx_int_t taskId = 0);
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t enableRWMutexTracking(sipx_int_t taskId = 0);
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t enableTimerTracking(sipx_int_t taskId = 0);
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t enableTaskTracking(sipx_int_t taskId = 0);
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t enableSocketTracking(sipx_int_t taskId = 0);
    //: Set trace flag, if non-zero, print a line of info

   static sipx_int_t disableTracking();
    //: Set trace flag, if non-zero, print a line of info

   static void addAllocCnt(sipx_int_t addr,
                                                  sipx_int_t taskId);
    //: Add allocCnt for the task, used for OsSocket tracking

   static void addAllocCnt(sipx_int_t size,
                                                  sipx_int_t addr,
                                                  sipx_int_t taskId);
    //: Add allocCnt for the task, used for memory/OsMutex/OsRWMutex tracking

        static void addAllocCnt(sipx_int_t size,
                                                          sipx_int_t addr,
                                                          const char* name, 
                                                          sipx_int_t pArg, 
                                                          sipx_int_t priority, 
                                                          sipx_int_t options,
                                                          sipx_int_t taskId);
    //: Add allocCnt for the task, used for OsTask tracking

        static void addAllocCnt(sipx_int_t state,
                                                          sipx_int_t addr,
                                                          sipx_int_t timerId, 
                                                          sipx_int_t type, 
                                                          sipx_int_t taskId);
    //: Add allocCnt for the task, used for OsTimer/OsCSem tracking

        static void addAllocCnt(sipx_int_t options,
                                                          sipx_int_t addr,
                                                          sipx_int_t state, 
                                                          sipx_int_t taskId);
    //: Add allocCnt for the task, used for OsBSem tracking

        static void addAllocCnt(sipx_int_t addr,
                                                          const char* name, 
                                                          sipx_int_t taskId);
    //: Add allocCnt for the task, used for OsMsgQ tracking

        static void addFreeCnt(sipx_int_t addr, sipx_int_t taskId = 0);
    //: Add freeCnt for the task

/* ============================ ACCESSORS ================================= */

   static void showMem(sipx_int_t taskId = 0);

   static sipx_int_t delta();
     //:Return the change to the number of outstanding memory allocations
     //:since the last checkpoint.

   static sipx_int_t delta(sipx_int_t taskId);
     //:Return the change to the number of outstanding memory allocations
     //:since the last checkpoint.

   static sipx_int_t allocCnt(sipx_int_t taskId = 0);
     //:Return the number of memory allocations (monotonically increasing)

   static sipx_int_t rscStatus();
     //:Return the number of memory frees (monotonically increasing)

   static sipx_int_t freeCnt(sipx_int_t taskId = 0);
     //:Return the number of memory frees (monotonically increasing)

   static sipx_int_t netAllocCnt();
     //:Return the net number of allocations (allocCnt - freeCnt)

   static sipx_int_t netAllocCnt(sipx_int_t taskId);
     //:Return the net number of allocations (allocCnt - freeCnt)

/* ============================ INQUIRY =================================== */

/* //////////////////////////// PROTECTED ///////////////////////////////// */
protected:

/* //////////////////////////// PRIVATE /////////////////////////////////// */
private:
        static sipx_long_t mCheckpoint; // Used to remember the net number of 
                     // allocations at time of checkpoint
        static OsMutex    *mpResourceStoreLock;

        static UtlRscStore mUtlRscStore;

        static sipx_int_t mTaskId;
};


/* ============================ INLINE METHODS ============================ */
#endif // RSC_TEST

#endif  // _UtlRscTrace_h_

