//
// Copyright (C) 2004, 2005 Pingtel Corp.
// 
//
// $$
////////////////////////////////////////////////////////////////////////
//////


// SYSTEM INCLUDES
#include <assert.h>

// APPLICATION INCLUDES
#include "os/OsDateTime.h"
#include "os/OsEventMsg.h"
#include "os/OsQueuedEvent.h"

// EXTERNAL FUNCTIONS
// EXTERNAL VARIABLES
// CONSTANTS
// STATIC VARIABLE INITIALIZATIONS

/* //////////////////////////// PUBLIC //////////////////////////////////// */

/* ============================ CREATORS ================================== */

// Constructor
OsQueuedEvent::OsQueuedEvent(OsMsgQ& rMsgQ, const sipx_int_t userData)
:  mUserData(userData),
   mpMsgQ(&rMsgQ)
{
}

// Destructor
OsQueuedEvent::~OsQueuedEvent()
{
   mpMsgQ = NULL;
   // no work required
}

/* ============================ MANIPULATORS ============================== */

// Set the event data and send an event message to the designated queue
// Return the result of the message send operation.
OsStatus OsQueuedEvent::signal(const sipx_int_t eventData)
{
   OsStatus res;

   res = doSendEventMsg(OsEventMsg::NOTIFY, eventData);
   return res;
}

// Set the user data value for this object
// Always returns OS_SUCCESS.
OsStatus OsQueuedEvent::setUserData(sipx_int_t userData)
{
   mUserData = userData;
   return OS_SUCCESS;
}

/* ============================ ACCESSORS ================================= */

// Return the user data specified when this object was constructed.
// Always returns OS_SUCCESS.
OsStatus OsQueuedEvent::getUserData(sipx_int_t& rUserData) const
{
   rUserData = mUserData;
   return OS_SUCCESS;
}

/* ============================ INQUIRY =================================== */

/* //////////////////////////// PROTECTED ///////////////////////////////// */

/* //////////////////////////// PRIVATE /////////////////////////////////// */

// Send an event message to the designated message queue.
// Return the result of the message send operation.
OsStatus OsQueuedEvent::doSendEventMsg(const sipx_int_t msgType,
                                       const sipx_int_t eventData) const
{
   OsTime timestamp;

   OsDateTime::getCurTimeSinceBoot(timestamp);

   OsEventMsg msg(msgType,     // event msg type
                  *this,       // this event
                  eventData,   // data signaled with event
                  timestamp);  // event timestamp (time since boot)

   OsStatus rc;
   if (mpMsgQ)
   {
      rc = mpMsgQ->send(msg);
   }
   else
   {
      rc = OS_FAILED;
   }
   return rc;
}

/* ============================ FUNCTIONS ================================= */


