//
// Copyright (C) 2004, 2005 Pingtel Corp.
// 
//
// $$
////////////////////////////////////////////////////////////////////////

#include <cppunit/TestCase.h>
#include <cppunit/SourceLine.h>
#include <cppunit/Asserter.h>
#include <string>

//Application Includes
#include <os/OsDefs.h>
#include <os/OsFileSystem.h>
#include <sipxunit/TestUtilities.h>
#include <os/OsTestUtilities.h>

void OsTestUtilities::createTestDir(OsPath& root)
{
    OsStatus stat;

    OsFileSystem::getWorkingDirectory(root);
    root.append(OsPath::separator).append("OsFileSystemTest");

    if (OsFileSystem::exists(root))
    {
        removeTestDir(root);
    }

    stat = OsFileSystem::createDir(root);
    CPPUNIT_ASSERT_MESSAGE("setup root test dir", stat == OS_SUCCESS);
}

void OsTestUtilities::removeTestDir(OsPath &root)
{
    OsStatus stat;
    if (OsFileSystem::exists(root))
    {
        stat = OsFileSystem::remove(root, TRUE, TRUE);
        CPPUNIT_ASSERT_MESSAGE("teardown root test dir", stat == OS_SUCCESS);
    }
}

void OsTestUtilities::initDummyBuffer(char *buff, sipx_int_t size)
{
    for (sipx_int_t i = 0; i < size; i++)
    {
        buff[i] = (char)(i % 256);
    }
}

UtlBoolean OsTestUtilities::testDummyBuffer(char *buff, sipx_ulong_t size, sipx_ulong_t position)
{
    for (sipx_ulong_t i = 0; i < size; i++)
    {
        char expected = (char)((position + i) % 256);
        if (buff[i] != expected)
        {
            printf("buff[%li] = %i, expected = %i\n", position, buff[i], expected);
            return false;
        }
    }

    return true;
}


OsStatus OsTestUtilities::createDummyFile(OsPath testFile, sipx_ulong_t size)
{
    OsStatus stat;
    char wbuff[10000];
    sipx_ulong_t wbuffsize = (sipx_ulong_t)sizeof(wbuff);

    OsTestUtilities::initDummyBuffer(wbuff, sizeof(wbuff));

    OsFile wfile(testFile);
    stat = wfile.open(OsFile::CREATE);
    if (stat == OS_SUCCESS)
    {
        sipx_ulong_t wposition = 0;
        for (sipx_int_t i = 0; stat == OS_SUCCESS && wposition < wbuffsize; i++)
        {
            sipx_ulong_t remaining = wbuffsize - wposition;
            sipx_ulong_t byteswritten = 0;
            stat = wfile.write(wbuff + wposition, remaining, byteswritten);
            wposition += byteswritten;
        }

        wfile.close();
    }

    return stat;
}

UtlBoolean OsTestUtilities::verifyDummyFile(OsPath testFile, sipx_ulong_t size)
{
    OsStatus stat;
    UtlBoolean ok = false;
    char rbuff[256];
    sipx_ulong_t rbuffsize = (sipx_ulong_t)sizeof(rbuff);
    OsFile rfile(testFile);
    stat = rfile.open();
    if (stat == OS_SUCCESS)
    {
        sipx_ulong_t rposition = 0;
        ok = true;
        for (sipx_int_t i = 0; ok && rposition < size; i++)
        {
            sipx_ulong_t remaining = (size - rposition);
            sipx_ulong_t readsize = remaining < rbuffsize ? remaining : rbuffsize;
            sipx_ulong_t bytesread = 0;
            stat = rfile.read(rbuff, readsize, bytesread);
            if (stat != OS_SUCCESS)
            {
                ok = false;
                printf("Error reading file, status = %i", stat);
            }
            else
            {
                ok = OsTestUtilities::testDummyBuffer(rbuff, bytesread, rposition);
                rposition += bytesread;
            }
        }

        rfile.close();
    }

    return ok;
}

