// 
// 
// Copyright (C) 2005-2006 SIPez LLC.
// Licensed to SIPfoundry under a Contributor Agreement.
// 
// Copyright (C) 2004-2006 SIPfoundry Inc.
// Licensed by SIPfoundry under the LGPL license.
// 
// Copyright (C) 2004-2006 Pingtel Corp.
// Licensed to SIPfoundry under a Contributor Agreement.
// 
// $$
//////////////////////////////////////////////////////////////////////////////


#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestCase.h>

#include <limits.h>
#include <string.h>
#include <stdlib.h>
#include <cstdarg>

#include <utl/UtlLongLongInt.h>
#include <utl/UtlString.h>
#include <sipxunit/TestUtilities.h>

using namespace std ;

/**  This class is used to test the UtlLongLongInt utility class. 
*
*    PLEASE READ THE README FILE THAT CAN FOUND IN THE SAME DIRECTORY AS
*    THIS FILE. The Readme describes the organization / flow of tests and
*    without reading this file, the following class (and all unit tests)
*    may not make a lot of sense and might be difficult to comprehend. 
*/
class UtlLongLongIntTests : public CppUnit::TestCase
{

    CPPUNIT_TEST_SUITE(UtlLongLongIntTests);
    CPPUNIT_TEST(testConstructor) ; 
    CPPUNIT_TEST(testCompareTo) ; 
    CPPUNIT_TEST(testCompareTo_NonIntll) ; 
    CPPUNIT_TEST(testEquals) ; 
    CPPUNIT_TEST(testEquals_NonIntll) ; 
    CPPUNIT_TEST(testSetValue) ; 
    CPPUNIT_TEST(testGetContainableType) ; 
    CPPUNIT_TEST(testOperators) ; 
    CPPUNIT_TEST_SUITE_END();

private:

    struct BasicIntllVerifier 
    {
        const char* message ; 
        INT64 input ; 
        INT64 expectedValue ; 
    } ;
    
    static const INT64 llsipx_int_t_Zero ;
    static const INT64 llsipx_int_t_Positive ;
    static const INT64 llsipx_int_t_Negative ;
    //LLONG_MAX
    //LLONG_MIN
    static const sipx_int_t INDEX_NOT_FOUND  ; 
    // An indication that the test should be ignored. 
    static const sipx_int_t IGNORE_TEST ; 
    
    static const BasicIntllVerifier commonTestSet[] ; 
    static const sipx_int_t commonTestSetLength ; 

    enum EqualOrCompareTest { TEST_EQUAL, TEST_COMPARE } ;
    
public:
    UtlLongLongIntTests()
    {
    }

    void setUp()
    {
    }
    
    void tearDown()
    {
    }

    ~UtlLongLongIntTests()
    {
    }

    /** Sandbox method for experimenting with the API Under Test. 
    *   This method MUST be empty when the test drivers are being
    *   checked in (final checkin) to the repository.
    */
    void DynaTest()
    {
    }

    /** Test the constructor
    *
    *    The test data for this test case is :-
    *     a) llsipx_int_t = 0
    *     b) llsipx_int_t = +ve sipx_longlong_teger
    *     c) llsipx_int_t = -ve sipx_longlong_teger
    *     d) llsipx_int_t = max +ve sipx_longlong_teger
    *     e) llsipx_int_t = max -ve sipx_longlong_teger.
    *     The above set of data will be referred to as the 
    *     Common Data Set henceforth! The common data set has been defined
    *     as a static const array of sipx_longlong_tegers and a sipx_short_t description about 
    *     the data is in a const array of char*
    */
    void testConstructor()
    {
        // First test the default constructor
        UtlLongLongInt testIntll ; 
        const char* msg0 = "Test the default constructor" ; 
        CPPUNIT_ASSERT_EQUAL_MESSAGE(msg0, 0LL, testIntll.getValue()) ; 

        // Now test the single argument constructor for each of 
        // the common test data type
        const char* prefix = "Test UtlLongLongInt(sipx_int_t value), where value = " ;
        for (sipx_int_t i = 0  ;i < commonTestSetLength; i++)
        { 
            string msg ;
            TestUtilities::createMessage(2, &msg, prefix, commonTestSet[i].message) ; 
            UtlLongLongInt testIntll(commonTestSet[i].input) ; 
            INT64 actualValue = testIntll.getValue() ; 
            CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.data(), commonTestSet[i].expectedValue, actualValue) ; 
        }
    }

    /** Test the compareTo method
    *
    *    The test data for this test is :-
    *        with the common data set, compare the data with
    *        another value that is a) equal to, b) greater than
    *        and c) less than itself
    */
    void testCompareTo()
    {
        utlTestCompareOrEquals(TEST_COMPARE) ; 
    }

    /** Test the isEqual(UtlContainable*) method. 
    *   
    *   The test data for this test is the same as the compareTo method. 
    */ 
    void testEquals()
    {
        utlTestCompareOrEquals(TEST_EQUAL) ; 
    }
    
    /* Test the isEqual method or the compareTo method.
    */
    void utlTestCompareOrEquals(EqualOrCompareTest testType)
    {
        const char* prefix = "";
        if (testType == TEST_COMPARE) 
        {
            prefix = "Test the compareTo method for a Long Long Integer whose value = " ; 
        }
        else if (testType == TEST_EQUAL) 
        {
            prefix = "Test the isEqual method for a Long Long Integer whose value = " ; 
        }
        const char* suffix1 = " - when value == arg"  ; 
        const char* suffix2 = " - when value > arg" ;
        const char* suffix3 = " - when value < arg" ; 
        string msg ;

        struct compareToData
        { 
            const char* message ; // Description about the type of test data
            INT64 baseValue ; // The value of the test UtlLongLongInt
            INT64 equalToBase ;  // A Long Long Integer that is equal to the base value
            INT64 greaterThanBase ; // Long Long Integer that is greater than the base value  
            INT64 lessThanBase ;  // Long Long Integer that is less than the base value
        } ; 
        const compareToData testData[] = { \
            { "zero", 0, 0, 10, -10 }, \
            { "positive integer", llsipx_int_t_Positive, llsipx_int_t_Positive, llsipx_int_t_Positive+10, llsipx_int_t_Positive-10 }, \
            { "negative integer", llsipx_int_t_Negative, llsipx_int_t_Negative, llsipx_int_t_Negative+10, llsipx_int_t_Negative-10 }, \
            { "integer at its maximum allowed value", LLONG_MAX, LLONG_MAX, IGNORE_TEST, LLONG_MAX-10 }, \
            { "integer at its minimum allowed value", LLONG_MIN, LLONG_MIN, LLONG_MIN+10, IGNORE_TEST } 
        } ; 
        const sipx_int_t testCount = sizeof(testData)/sizeof(testData[0]) ; 
        INT64 expectedForEquals = 0 ;
        INT64 expectedForGreaterThan = 1 ; 
        INT64 expectedForLessThan = -1 ; 
        
        // Loop to iterate through the array of test data. 
        for (sipx_int_t i = 0 ; i < testCount ; i++)
        {            
            UtlLongLongInt testIntll(testData[i].baseValue) ; 
            UtlLongLongInt llsipx_int_tForCompare ; 

            // -------------------------------------------------------------------------
            // first test the case when the rhs's llsipx_int_t = base llsipx_int_t. 
            llsipx_int_tForCompare = UtlLongLongInt(testData[i].equalToBase) ; 
            TestUtilities::createMessage(3, &msg, prefix, \
                                             testData[i].message, suffix1) ; 
            if (testType == TEST_COMPARE) 
            {
                INT64 actual = testIntll.compareTo(&llsipx_int_tForCompare) ;  
                CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.data(), expectedForEquals, actual) ; 
            }
            else if (testType == TEST_EQUAL) 
            {
                UtlBoolean actual = testIntll.isEqual(&llsipx_int_tForCompare) ; 
                CPPUNIT_ASSERT_MESSAGE(msg.data(), actual) ; 
            }
            // -------------------------------------------------------------------------

            // -------------------------------------------------------------------------
            // Now test the case where the llsipx_int_t's value is greater than the argument's value
            // In this case we SHOULD not try to compare for a llsipx_int_t which is at the 
            // MINIMUM value that any llsipx_int_t can assume as such a llsipx_int_t cannot be greater
            // than anything else. This is bound to cause overruns thus yielding unexpected results. 
            if (testData[i].baseValue != LLONG_MIN)
            {
                llsipx_int_tForCompare = UtlLongLongInt(testData[i].lessThanBase) ; 
                TestUtilities::createMessage(3, &msg, prefix, \
                                             testData[i].message, suffix2) ; 
                if (testType == TEST_COMPARE) 
                {
                    INT64 actual = testIntll.compareTo(&llsipx_int_tForCompare) ; 
                    CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.data(), expectedForGreaterThan, actual)  ; 
                }
                else if (testType == TEST_EQUAL) 
                {
                    UtlBoolean actual = testIntll.isEqual(&llsipx_int_tForCompare) ; 
                    CPPUNIT_ASSERT_MESSAGE(msg.data(), !actual) ; 
                }
            }
            // -------------------------------------------------------------------------
            
            // -------------------------------------------------------------------------
            // Now test the case where the integer's value is less than the argument's value
            // In this case we SHOULD not try to compare for an integer which is at the 
            // MAXIMUM value that any integer can assume as such an integer cannot be less than
            // anything else. This is bound to cause overruns thus yielding unexpected results. 
            if (testData[i].baseValue != LLONG_MAX)
            {
                llsipx_int_tForCompare = UtlLongLongInt(testData[i].greaterThanBase) ; 
                TestUtilities::createMessage(3, &msg, prefix, \
                                             testData[i].message, suffix3) ; 
                if (testType == TEST_COMPARE) 
                {
                    INT64 actual = testIntll.compareTo(&llsipx_int_tForCompare) ; 
                    CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.data(), expectedForLessThan, actual) ;
                }
                else if (testType == TEST_EQUAL) 
                {
                    UtlBoolean actual = testIntll.isEqual(&llsipx_int_tForCompare) ; 
                    CPPUNIT_ASSERT_MESSAGE(msg.data(), !actual) ; 
                }
            }
            // -------------------------------------------------------------------------
        } // End of loop to iterate through test data. 
    }

    /*!a Test the compareTo method when a non-UtlLongLongInt is passed. 
    *    
    */
    void testCompareTo_NonIntll()
    {
        UtlString testUtlString("Test String") ; 
        UtlLongLongInt testUtlLongLongInt(LLONG_MAX) ; 
        INT64 actual = testUtlLongLongInt.compareTo(&testUtlString) ; 
        // If a collectable is *NOT* an Integer, then the only thing that is predictible is
        // that this is not equal to the argument
        CPPUNIT_ASSERT_MESSAGE( "Compare a Long Long Integer with a String ", (actual != 0 )) ; 
    }

    /*!a Test the Equals Method when the argument passed is not a UtlLongLongInt
    *
    */
    void testEquals_NonIntll()
    {
        const char* prefix = "Test the isEquals(UtlContainable other) for an Integer whose " \
                             "value is " ; 
        const char* suffix = ", where other is a UtlContainable object that is not a " \
                             "UtlLongLongInt" ; 
        UtlString testString("This is a test string") ;
        for (sipx_int_t i = 0 ; i < commonTestSetLength; i++)
        {
            UtlLongLongInt testIntll(commonTestSet[i].input) ; 
            string msg ; 
            UtlBoolean isEqual = testIntll.isEqual(&testString);
            TestUtilities::createMessage(3, &msg, prefix, commonTestSet[i].message, suffix) ; 
            CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.data(), (UtlBoolean)false, isEqual) ;  
        }

    } //testEquals_NonIntll
    
    /*!a Test the setValue(llsipx_int_t) method. 
    * 
    *    Test Data :- 
    *        For each type in the common dataset as the base
    *        integer, add each type of common dataset llsipx_int_t
    */  
    void testSetValue()
    {
        const char* prefix = "Test the setValue(llsipx_int_t setter) method for a UtlLongLongInt " \
                             "whose value is " ; 
        const char* suffix1 = " -- verify that the previous value is returned " ;         
        const char* suffix2 = " -- verify that the new value has been set " ; 
        string msg ; 
        
        // For each type of UtlLongLongInt, verify that you can set 
        // every common type of Long Long Integer. 
        for (sipx_int_t i = 0 ; i < commonTestSetLength ; i++)
        {
            // Loop to test setting each common type of llsipx_int_t
            // for any given type of llsipx_int_t
            for (sipx_int_t j = 0 ; j < commonTestSetLength ; j++)
            {
                UtlLongLongInt baseIntll(commonTestSet[i].input) ; 

                INT64 oldActualValue = baseIntll.setValue(commonTestSet[j].input) ; 
                INT64 newActualValue = baseIntll.getValue() ; 
                
                // Verify that the return value = previous value
                TestUtilities::createMessage(5, &msg, prefix, commonTestSet[i].message, \
                                               " where setter = ", commonTestSet[j].message, \
                                               suffix1) ; 
                CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.data(), commonTestSet[i].expectedValue, \
                                                       oldActualValue) ; 
                
                // Verify that the value has been set.
                TestUtilities::createMessage(5, &msg, prefix, commonTestSet[i].message, \
                                               " where setter = ", commonTestSet[j].message, \
                                               suffix2) ; 
                CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.data(), commonTestSet[j].expectedValue, \
                                                       newActualValue) ; 
                               
            } 
        }
    } //testSetValue

    /*!a Test the getContainableType method
    *
    *    Test data = common test data set
    */
    void testGetContainableType()
    {
        const char* prefix = "Test the getContainableType() method for a UtlLongLongInt " \
                             "whose value is " ;
        string msg ; 
        for (sipx_int_t i = 0 ; i < commonTestSetLength; i++)
        {
            UtlLongLongInt testIntll(commonTestSet[i].input) ; 
            UtlContainableType actual = testIntll.getContainableType() ; 
            TestUtilities::createMessage(2, &msg, prefix, commonTestSet[i].message) ; 
            CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.data(), string("UtlLongLongInt"), \
                                      string(actual)) ; 
        }
    } //testGetContainableType

    void testOperators()
    {
        UtlLongLongInt testIntll(1);
        CPPUNIT_ASSERT((++testIntll).getValue() == 2);
        CPPUNIT_ASSERT((testIntll++).getValue() == 2);
        CPPUNIT_ASSERT((--testIntll).getValue() == 2);
        CPPUNIT_ASSERT((testIntll--).getValue() == 2);

        // test conversion operator
        UtlLongLongInt testIntll2(LLONG_MAX);
        CPPUNIT_ASSERT(testIntll2 == LLONG_MAX);
    } //testOperators
};


// ------------------- Static constant initializers -------------------------
const INT64 UtlLongLongIntTests::llsipx_int_t_Zero = 0LL ;
const INT64 UtlLongLongIntTests::llsipx_int_t_Positive = 101LL ;
const INT64 UtlLongLongIntTests::llsipx_int_t_Negative = -51LL ;
const UtlLongLongIntTests::BasicIntllVerifier \
      UtlLongLongIntTests::commonTestSet[]  = { \
         {"Zero", llsipx_int_t_Zero, llsipx_int_t_Zero},  \
         {"Positive Long Long Integer", llsipx_int_t_Positive, llsipx_int_t_Positive}, \
         {"Negative Long Long Integer", llsipx_int_t_Negative, llsipx_int_t_Negative}, \
         {"MAX VALUE Positive Long Long Integer", LLONG_MAX, LLONG_MAX}, \
         {"MIN VALUE Negative Long Long Integer", LLONG_MIN, LLONG_MIN} \
      } ; 

const sipx_int_t UtlLongLongIntTests::commonTestSetLength = 5 ; 
const sipx_int_t UtlLongLongIntTests::INDEX_NOT_FOUND = -1 ; 
const sipx_int_t UtlLongLongIntTests::IGNORE_TEST=-1 ; 
CPPUNIT_TEST_SUITE_REGISTRATION(UtlLongLongIntTests);
